% file MASS/corresp.d
% copyright (C) 1994-9 W. N. Venables and B. D. Ripley
%
\name{corresp}
\alias{corresp}
\title{
Simple Correspondence Analysis
}
\description{
Find the principal canonical correlation and corresponding row- and
column-scores from a correspondence analysis of a two-way contingency
table.
}
\usage{
corresp(tabl, nf=1, \dots)
}
\arguments{
\item{tabl}{
The function is generic, accepting various forms of the principal
argument for specifying a two-way frequency table.  Currently accepted
forms are matrices, data frames (coerced to frequency tables), objects
of class \code{crosstabs} and formulae of the form \code{~ F1 + F2}, where \code{F1}
and \code{F2} are factors.
}
\item{nf}{
The number of factors to be computed. Note that although 1 is the most
usual, one school of thought takes the first two singular vectors for
a sort of biplot.
}
\item{...}{
If the principal argument is a formula, a data frame may be specified
as well from which variables in the formula are preferentially
satisfied.
}}
\value{
An list object of class \code{correspondence} for which \code{print}, \code{plot} and
\code{biplot} methods are supplied.  The main components are the canonical
correlation(s) and the row and column scores. 
}
\details{
See the reference.  The \code{plot} method produces a graphical
representation of the table if \code{nf=1}, with the \emph{areas} of circles
representing the numbers of points.  If \code{nf} is two or more the
\code{biplot} method is called, which plots the second and third columns of
the matrices \code{A = Dr^(-1/2\}) U L} and \code{B = Dc^(-1/2) U V} where the
singular value decomposition is \code{U L V}.  Thus the x-axis is the
canonical correlation times the row and column scores. Although this
is called a biplot, it does \emph{not} have any useful inner product
relationship between the row and column scores.  Think of this as an
equally-scaled plot with two unrelated sets of labels.
}
\references{
Venables \& Ripley (1997), chapter 13.
}
\seealso{
\code{\link{svd}}, \code{\link{princomp}}
}
\examples{
data(quine)
ct <- corresp(~ Age + Eth, data=quine)
ct
\dontrun{plot(ct)}

data(caith)
library(mva)
biplot(corresp(caith, nf=2))
}
\keyword{category}
\keyword{multivariate}
% Converted by Sd2Rd version 0.3-2.
